unit UBrainFuck;

{$mode Delphi}{$H+}

interface

uses
  Classes, SysUtils, Dialogs;

type

  { TBrainFuck }

  TBrainFuck = class
    Source: TStringList;
    Output: TStringList;
    procedure Compile;
  end;

  { TBrainFuckInterpreter }

  TBrainFuckInterpreter = class
  private
    procedure Write(Value: Byte);
    function Read: Byte;
    function ReadCode: Char;
  public
    Source: string;
    SourcePosition: Integer;
    Memory: array of Byte;
    MemoryPosition: Integer;
    Loop: array of Integer;
    LoopCurrent: Integer;
    Output: string;
    Input: string;
    InputPosition: Integer;
    procedure Reset;
    procedure SingleStep;
    procedure Run;
    constructor Create;
    destructor Destroy; override;
  end;


implementation

resourcestring
  SProgramLowerLimit = 'Program run over lower limit';
  SProgramUpperLimit = 'Program run over upper limit';

{ TBrainFuckInterpreter }

procedure TBrainFuckInterpreter.Write(Value: Byte);
begin
  Output := Output + Char(Value);
end;

function TBrainFuckInterpreter.Read: Byte;
var
  Character: string;
begin
  Character := Copy(Input, InputPosition, 1);
  Result := Ord(Character[1]);
  Inc(InputPosition);
end;

function TBrainFuckInterpreter.ReadCode: Char;
var
  Code: string;
begin
  Code := Copy(Source, SourcePosition + 1, 1);
  Result := Code[1];
end;

procedure TBrainFuckInterpreter.Reset;
var
  I: Integer;
begin
  SourcePosition := 0;
  InputPosition := 0;
  Output := '';
  MemoryPosition := 0;
  for I := 0 to Length(Memory) - 1 do
    Memory[I] := 0;
  SetLength(Loop, 0);
end;

procedure TBrainFuckInterpreter.SingleStep;
var
  CodeText: string;
  Code: Char;
  C: Integer;
begin
  case ReadCode of
    '>': Inc(MemoryPosition);
    '<': if MemoryPosition > 0 then Dec(MemoryPosition)
      else raise Exception.Create(SProgramLowerLimit);
    '+': Memory[MemoryPosition] := Memory[MemoryPosition] + 1;
    '-': Memory[MemoryPosition] := Memory[MemoryPosition] - 1;
    '.': Write(Memory[MemoryPosition]);
    ',': Memory[MemoryPosition] := Read;
    '[': begin
      if Memory[MemoryPosition] = 0 then begin
        C := 1;
        Inc(SourcePosition);
        while C > 0 do begin
          case ReadCode of
            '[': Inc(C);
            ']': Dec(C);
          end;
          Inc(SourcePosition);
        end;
        Dec(SourcePosition);
      end;
    end;
    ']': begin
      if Memory[MemoryPosition] > 0 then begin
        C := 1;
        Dec(SourcePosition);
        while C > 0 do begin
          case ReadCode of
            ']': Inc(C);
            '[': Dec(C);
          end;
          Dec(SourcePosition);
        end;
      end;
    end;
  end;
  Inc(SourcePosition);
end;

procedure TBrainFuckInterpreter.Run;
begin
  Reset;
  while SourcePosition < Length(Source) do
    SingleStep;
end;

constructor TBrainFuckInterpreter.Create;
begin
  SetLength(Memory, 32000);
end;

destructor TBrainFuckInterpreter.Destroy;
begin
  inherited Destroy;
end;

{ TBrainFuck }

procedure TBrainFuck.Compile;
begin

end;

end.

